<?php

declare(strict_types=1);

namespace Drupal\pdf_reducer\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\File\FileExists;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\ByteSizeMarkup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\Plugin\Validation\Constraint\BaseFileConstraintValidator;
use Drupal\pdf_reducer\PdfReducer;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;

/**
 *
 */
class PdfReducerConstraintValidator extends BaseFileConstraintValidator implements ContainerInjectionInterface {

  use StringTranslationTrait;

  public function __construct(
    protected MessengerInterface $messenger,
    protected LoggerInterface $logger,
    protected FileSystemInterface $fileSystem,
    protected PdfReducer $pdfReducer,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): PdfReducerConstraintValidator {
    return new static(
      $container->get('messenger'),
      $container->get('logger.factory')->get('pdf_reducer'),
      $container->get('file_system'),
      $container->get('pdf_reducer')
    );
  }

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function validate(mixed $value, Constraint $constraint): void {
    assert($constraint instanceof PdfReducerConstraint);
    $file = $this->assertValueIsFile($value);
    if ($file->getMimeType() !== 'application/pdf') {
      return;
    }
    $previous_size = $file->getSize();
    $file_uri = $file->getFileUri();
    try {
      $processed_file = $this->pdfReducer->reduce($file);
    }
    catch (\Exception $e) {
      $this->logger->notice("Could not compress the PDF file " . $file->getFilename() . ":\n" . $e->getMessage());
      return;
    }
    $tmp_file_uri = $processed_file->getFileUri();
    $new_size = $processed_file->getSize();
    if ($new_size < $previous_size) {
      if (!isset($file_uri, $tmp_file_uri, $previous_size, $new_size)) {
        $this->logger->notice("Error when trying to compress this PDF file: " . print_r($file, TRUE));
        return;
      }
      $this->fileSystem->move($tmp_file_uri, $file_uri, FileExists::Replace);
      $this->messenger->addStatus($this->t('For better download performance and ecological footprint, the uploaded PDF was processed to have a lower file size (from %before to %after).',
        [
          '%before' => ByteSizeMarkup::create($previous_size),
          '%after' => ByteSizeMarkup::create($new_size),
        ]));
    }
  }

}
